//日期格式化
Date.prototype.format = function(fmt) {
    const o = {
        "M+" : this.getMonth()+1,                 //月份
        "d+" : this.getDate(),                    //日
        "h+" : this.getHours(),                   //小时
        "m+" : this.getMinutes(),                 //分
        "s+" : this.getSeconds(),                 //秒
        "q+" : Math.floor((this.getMonth()+3)/3), //季度
        "S"  : this.getMilliseconds()             //毫秒
    };
    if(/(y+)/.test(fmt)) {
        fmt=fmt.replace(RegExp.$1, (this.getFullYear()+"").substr(4 - RegExp.$1.length));
    }
    for(const k in o) {
        if(new RegExp("("+ k +")").test(fmt)){
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length)));
        }
    }
    return fmt;
}

//获取农历的月日
Date.prototype.getChinese = function(){
    const arr = this.toLocaleString('ja-JP-u-ca-chinese').split('-');
    let month = parseInt(arr[1]);
    let day = parseInt(arr[2]);
    if(month < 10){
        month = '0' + month;
    }
    if(day < 10){
        day = '0' + day;
    }
    return month + '' + day;
}

//获取2个时间之间相差多少天
function getPassDay(day1,day2){
    const season = getSeason();
    const rate = season === '春' ? 2 : season === '冬' ? 0 : 1;
    return parseInt((day2 - day1) * rate / (1000 * 60 * 60 * 24))
}

//获取2个时间之间相差多少分钟
function getPassMin(day1,day2){
    return parseInt((day2 - day1) / (1000 * 60))
}

//给指定时间增加指定天数
function addDay(date,num){
    return new Date(date.getTime() + (num * 1000 * 60 * 60 * 24))
}

//给指定时间增加指定分钟
function addMin(date,num){
    return new Date(date.getTime() + (num * 1000 * 60))
}

//打乱数组
if (!Array.prototype.derangedArray) {
    Array.prototype.derangedArray = function() {
        for(let j, x, i = this.length; i; j = parseInt(Math.random() * i), x = this[--i], this[i] = this[j], this[j] = x);
        return this;
    };
}

//生成从minNum到maxNum的随机数
function randomNum(minNum,maxNum){
    switch(arguments.length){
        case 1:
            return parseInt(Math.random()*minNum+1,10);
            break;
        case 2:
            return parseInt(Math.random()*(maxNum-minNum+1)+minNum,10);
            break;
        default:
            return 0;
            break;
    }
}

//获取数组中的随机一个值
Array.prototype.getRandom = function(num = 1){
    if(num === 1){
        return this[randomNum(0, this.length - 1)];
    }
    if(num > this.length){
        num = this.length;
    }
    const arr = [];
    const indexs = [];
    for(let i = 0 ; i < num ; i++){
        let index = randomNum(0, this.length - 1);
        while (indexs.indexOf(index) !== -1){
            index = randomNum(0, this.length - 1);
        }
        indexs.push(index);
        arr.push(this[index])
    }
    return arr;
}

//是否包含指定集合里所有元素
Array.prototype.hasAll = function(arr = []){
    return !(arr.filter(e => this.indexOf(e) === -1).length > 0)
}

//保留两位小数的解析浮点型
function parseFloat2Fiexd(val){
    return parseFloat(parseFloat(val).toFixed(2));
}

//数组去重
function arrayDeweight(arr){
    return [...new Set(arr)]
}

//多个数组取交集
function arrayIntersection(arr){
    return arr.reduce((a, b) => a.filter(c => b.includes(c)));
}

/**
 * 计算百分比
 * @param   {number} num   分子
 * @param   {number} total 分母
 * @returns {number} 返回数百分比
 */
function Percentage(num, total) {
    if (num == 0 || total == 0){
        return 0;
    }
    return (Math.round(num / total * 10000) / 100.00);// 小数点后两位百分比
}

//创建4位ID
function S4() {
    return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
}

//创建32位ID
function getNewId() {
    return ""+S4()+S4()+S4()+S4()+S4()+S4()+S4()+S4();
}

//localStorage封装工具,自动进行json格式转换
const Store = {
    get : function(key){
        let value = localStorage.getItem(key);
       /* $.ajax({
            url: "temp/" + key + ".saved",
            async: false,
            success(resp){
                value = resp
            }
        });*/
        try {
            value = JSON.parse(value);
        }catch (e) {}
        return value;
    },
    set : function(key,value){
        let val = null;
        try {
            val = JSON.stringify(value);
            localStorage.setItem(key,val || value);
        }catch (e) {}
    },
    remove : function(key){
        if(window.control){
            window.control.del((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName);
        }else{
            localStorage.removeItem(key);
        }
    }
};

//存档
let needSave = false;
let isDie = false;
let canSaveExit = true;
function save(name = conf.savedName, saveData = data){
    canSaveExit = false;
    //成就判定
    checkAchievement();

    //每次要存档时，记录当前世界BOSS的属性
    let finalBoss = null;
    const map = mapCfgs.filter(e => e.name === data.world.name);
    if(map.length > 0){
        finalBoss = map[0].finalBoss;
    }
    if(data.battleMonster && data.battleMonster.name === finalBoss){
        data.world.finalBossObj = data.battleMonster;
    }

    if(name !== conf.savedName){
        let val = null;
        try {
            val = JSON.stringify(saveData);
        }catch (e) {}
        try {
            window.control.save(`${(diff === '正常难度' || diff == '0' ? '' : diff) + name}###---###${val || saveData}`);
        }catch (e) {
            if(window.control){
                msg('存档失败：' + e)
            }
            Store.set((diff === '正常难度' || diff == '0' ? '' : diff) + name,saveData);
        }
    }else{
        needSave = true;
    }
}

//自动存档
setInterval(function(){
    if(needSave && !isDie){
        needSave = false;

        const {worlds, ...datas} = data;
        if(window.control){
            let val = null;
            try {
                val = JSON.stringify(datas);
            }catch (e) {}
            window.control.save(`${(diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName}###---###${val || datas}`);
            canSaveExit = true;
        }else{
            Store.set((diff === '正常难度' || diff == '0' ? '' : diff) + savedName,datas);
            msg('自动存档成功', false);
        }
        systemLock = false;
    }
},2000)

//安全退出
$('.safe-exit').click(function(){
    if(!window.control){
        return;
    }
    $('.alert-win').show().find('.alert-win-title').html('系统正在安全退出，请稍等……')
    setInterval(function(){
        if(canSaveExit){
            window.control.exit();
        }
    },500);
});

//加载
function load(name = conf.savedName){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + name);
        return resp ? JSON.parse(resp) : null;
    }else{
        return Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + savedName);
    }
}

//加载配置
function loadConf(){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName);
        return new Conf(resp ? JSON.parse(resp) : {});
    }
    if(Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName)){
        return new Conf(Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + confSavedName));
    }

    return new Conf({});
}

//加载世界
function loadWorlds(){
    if(window.control){
        const resp = window.control.load((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map");
        data.worlds = resp ? JSON.parse(resp) : [];
    }else{
        data.worlds = Store.get((diff === '正常难度' || diff == '0' ? '' : diff) + conf.savedName + "-map") || [];
    }
}

//加载世界
function saveWorlds(){
    save(conf.savedName + '-map', data.worlds)
}

//加载难度
function loadDiff(){
    return window.control ? window.control.load("diff") : Store.get("diff");
}

//加载难度
function saveDiff(){
    window.control ?  window.control.save(`diff###---###${diff || '0'}`) : Store.set("diff", diff || '0');
}

//批量创建物品
function getBatchItem(arr = [], obj){
    let data = {};
    arr.forEach(e => {
        if(typeof obj === 'function'){
            data = {
                ...data,
                ...obj(e)
            }
        }else{
            data[e] = obj;
        }
    })
    return data;
}

//以下为作者调试用的方法

//获取指定地图可以采集到的所有物品
function getAllElesByMapName(name){
    let arr = [];
    terrainArray[name].forEach(e => {
        Object.keys(terrainElements[e].elements).forEach(el => {
            arr = [...arr, ...elementCfgs[el].getItem.map(i => i.name)]
        })
    })

    return [...new Set(arr)];
}
//获取指定地图可以打怪掉落的所有物品
function getAllMonsterItemByMapName(name){
    const level = mapCfgs.filter(m => m.name === name)[0].level;
    let arr = [];
    terrainArray[name].forEach(e => {
        Object.keys(terrainElements[e].monster).filter(el => monsterCfgs[el].level >= level[0] && monsterCfgs[el].level <= level[1]).forEach(el => {
            arr = [...arr, ...monsterCfgs[el].items.map(i => i.name)]
        })
    })

    return [...new Set(arr.filter(e => !e.startsWith('图纸')))];
}

//点亮全部地图
function showAllMap(){
    for(let y = 0 ; y < ynum ; y++){
        for(let x = 0 ; x < xnum ; x++){
            data.world.showMap[`${x},${y}`] = 1;
        }
    }
}
