//创建NPC配置
function createNpcCfg(props = {}){
    return {
        gold : [100,1000],//随机的金币
        //可以随机到的职业
        profession : ['剑士','骑士'/*,'吟游诗人','牧师','游侠','盗贼','猎魔人','野蛮人','奥术师','圣骑士','预言家','舞娘'*/],
        ...props,
    }
}

//NPC配置
const npcCfg = {
    minNum : 20,//每个地图NPC的最小数量，小于这个数量就会刷新出新的NPC
    maxNum : 50,//每个地图NPC的最大数量，达到或者超过这个数量就不会刷新NPC了
    leaveEnterRate : 10,//NPC移动时直接进入或离开地图的概率，百分之
    //各个类型的NPC
    types : {
        行人 : createNpcCfg({
            textList : [
                '据说有其他世界的入侵者来到我们的世界了，也不知道外面世界的人长什么样。',
                '野外的道路真难走。',
                '你看起来很面生啊，难道你就是外面世界的入侵者？',
                '在野外行走要小心怪物的偷袭。',
                '入侵者什么的跟我无关，反正有帝国在上面顶着呢。',
                '你有什么事吗？',
            ],
        }),
        商人 : createNpcCfg({
            gold : [500,5000],//随机的金币
            textList : [
                '该死的，那些佣兵要价太高了。',
                '哎，这年头生意也不好做啊。',
                '你听说入侵者的事情了吗？也不知道来了多少人。',
                '因为入侵者，现在到处都戒严了，生意都变差了。',
                '我一定要成为欧伦斯最伟大的商人！',
            ],
        }),
        猎人 : createNpcCfg({
            textList : [
                '猎人，一不小心就会变成猎物。',
                '你看到我的猎犬了吗？',
                '听说欧伦斯全城戒严了，因为有异世界的入侵者来到了我们的世界。',
                '为什么召集猎人追查入侵者的任务不让我参与呢，真是的。',
                '啊哈，这一趟真是大丰收啊！',
                '我的目标和那些没实力的猎人可不一样，我的目标是……哼，关你什么事。',
                '听说实力足够强的猎人都接到了追查入侵者的任务，我还需要努力啊。',
            ],
        }),
    },
};

//初始化本地图的NPC
function initNpc(){
    if(!data.inDungeon && data.worldIndex >= 8){
        if(data.world.npcs){
            data.world.npcs = null;
        }
        return;
    }
    if(!data.inDungeon && !data.world.npcs){
        data.world.npcs = [];
        data.world.npcDate = data.date.format('yyyy-MM-dd');
        //类型列表
        const types = Object.keys(npcCfg.types);
        //随机数量
        const num = randomNum(npcCfg.minNum, npcCfg.maxNum);
        for(let i = 0 ; i < num ; i++){
            const type = types.getRandom();
            data.world.npcs.push(createNewNpc(type));
        }
    }
    if(data.world.npcs){
        let num = data.worldIndex + 1;
        num *= num;
        data.world.npcs.forEach(e => {
            if(!e.createDay){
                e.createDay = data.date.getTime()
            }
            const number = randomNum(1,100);
            if(e.rank == null){
                e.rank = number <= 1 ? '神话' : number <= 10 ? '传说' : number <= 30 ? '精英' : '';
            }
            if(e.gold == null){
                e.gold = randomNum(npcCfg.types[e.type].gold[0], npcCfg.types[e.type].gold[1]) * num;
            }
            if(!e.profession){
                e.profession = npcCfg.types[e.type].profession.getRandom();
            }
        })
    }
}

//创建一个NPC
function createNewNpc(type){
    let num = data.worldIndex + 1;
    num *= num;
    const number = randomNum(1,100);
    return {
        id : getNewId(),
        name : `${type}${npcNames.getRandom()}`,
        type,
        gold : randomNum(npcCfg.types[type].gold[0], npcCfg.types[type].gold[1]) * num,
        rank : number <= 1 ? '神话' : number <= 10 ? '传说' : number <= 30 ? '精英' : '',
        createDay : data.date.getTime(),
        text : npcCfg.types[type].textList.getRandom(),
        profession : npcCfg.types[type].profession.getRandom(),
        location : [randomNum(0, xnum - 1),randomNum(0, ynum - 1)]
    };
}

//点击NPC
$('.body').on('click','.npc', function(){
    const id = $(this).attr('data-id');
    const npc = data.world.npcs.filter(e => e.id === id)[0];
    $('.npc-win .npc-name').html(npc.name);
    $('.npc-win p').html(npc.text);
    $('.npc-win-modal').show();
    const win = $('.npc-win').show().attr('data-id', id);
})

//NPC移动
function npcMove(){
    if(!data.inDungeon && data.world.npcs && data.world.npcDate !== data.date.format('yyyy-MM-dd')){
        data.world.npcDate = data.date.format('yyyy-MM-dd')
        //NPC离开
        data.world.npcs = data.world.npcs.filter(e => {
            if(data.world.npcs.length <= npcCfg.minNum){
                return true;
            }
            if(data.world.npcs.length >= npcCfg.maxNum){
                return false;
            }
            return randomNum(0,100) > npcCfg.leaveEnterRate;
        })

        //NPC移动
        data.world.npcs.forEach(e => {
            e.location = getMoveLocation(e.location);
            //判断本次移动是采集还是战斗
            const terrainElement = terrainElements[data.world.map[e.location[1]][e.location[0]]];
            if(terrainElement){
                const number = randomNum(1,100);
                if(!e.items){
                    e.items = {};
                }

                if(number <= 50){//采集
                    if(terrainElement.elements){
                        const {canGather, num, getItem} = elementCfgs[Object.keys(terrainElement.elements).getRandom()];
                        if(canGather){
                            //循环采集次数
                            for(let i = 0 ; i < num ; i++){
                                //循环采集物品
                                getItem.forEach(i => {
                                    const {name, minNum, maxNum} = i;
                                    const num = randomNum(minNum, maxNum);
                                    if(num > 0){
                                        if(!e.items[name]){
                                            e.items[name] = 0;
                                        }
                                        e.items[name] += num;
                                    }
                                })
                            }
                        }
                    }
                }else{//战斗
                    if(terrainElement.monster){
                        const monsters = Object.keys(terrainElement.monster).filter(m => npcCanKillMonster(e.rank, m));
                        if(monsters.length > 0){
                            const {items} = monsterCfgs[monsters.getRandom()];
                            //循环掉落物
                            items.forEach(i => {
                                const {name, minNum, maxNum} = i;
                                const num = randomNum(minNum, maxNum);
                                if(num > 0){
                                    if(!e.items[name]){
                                        e.items[name] = 0;
                                    }
                                    e.items[name] += num;
                                }
                            })
                        }
                    }
                }
            }
        })

        //类型列表
        const types = Object.keys(npcCfg.types);
        //NPC加入
        for(let i = data.world.npcs.length ; i <= npcCfg.maxNum ; i++){
            if(data.world.npcs.length >= npcCfg.maxNum){
                continue;
            }
            if(data.world.npcs.length < npcCfg.minNum || randomNum(1,100) <=  npcCfg.leaveEnterRate){
                data.world.npcs.push(createNewNpc(types.getRandom()));
            }
        }
    }
}

//校验怪物是否可以被这个NPC击杀
function npcCanKillMonster(npcType, monster){
    if(monsterCfgs[monster].level >= data.world.level[0] - (monsterCfgs[monster].type ? 10 : 0) && monsterCfgs[monster].level <= data.world.level[1]){
        if(npcType === '神话'){
            return true;
        }
        if(npcType === '传说'){
            return monsterCfgs[monster].type !== '神话';
        }
        if(npcType === '精英'){
            return monsterCfgs[monster].type !== '神话' && monsterCfgs[monster].type !== '精英';
        }
        return !monsterCfgs[monster].type;
    }
    return false;
}


//获取移动后的位置
function getMoveLocation(location){
    const [x,y] = location;
    //存放可前往的坐标
    const xarray = [];
    const yarray = [];
    if(x > 0){
        xarray.push(x - 1);
    }
    if(x < xnum - 1){
        xarray.push(x + 1);
    }
    if(y > 0){
        yarray.push(y - 1);
    }
    if(y < ynum - 1){
        yarray.push(y + 1);
    }

    //1则移动x坐标，2则移动y坐标
    const flag = randomNum(1,2);
    if(flag === 1){
        return [xarray.getRandom(), y];
    }else{
        return [x, yarray.getRandom()];
    }
}

//点击交易
$('.npc-shop-menu').click(function(){
    renderNpcShop();
});

//渲染NPC交易
function renderNpcShop(){
    const id = $('.npc-win').attr('data-id');
    if(id){
        $('.npc-shop-win-items').html('');
        const npc = data.world.npcs.filter(e => e.id === id)[0];
        $('.your-gold').html(data.gold || 0);
        $('.npc-gold').html(npc.gold);
        const active = $('.npc-shop-win .active').html();
        if(active === '购买'){
            if(npc.items && Object.keys(npc.items).length > 0){
                let html = ``;
                Object.keys(npc.items).forEach(e => {
                    const gold = parseInt(elementCfgs[e].gold * 0.8) || 1;
                    html += `<div class="npc-shop-win-item">`;
                    html += `<h5>${e}<span class="to-cybernet" data-name="${e}">🔍</span> * ${npc.items[e]}</h5>`;
                    html += `<div class="npc-price">售价：${gold}金币</div>`;
                    html += `<button class="buy-npc-item" data-name="${e}" data-id="${id}" data-num="1">买1个</button>`;
                    html += `<button class="buy-npc-item" data-name="${e}" data-id="${id}" data-num="10">买10个</button>`;
                    html += `<button class="buy-npc-item" data-name="${e}" data-id="${id}" data-num="1000000">买全部</button>`;
                    html += `</div>`;
                })
                $('.npc-shop-win-items').append(html)
            }
        }else{
            let html = '';
            data.bag.forEach(e => {
                if(data.armor && data.armor.id === e.id){
                    return;
                }
                if(data.bagEquip && data.bagEquip.id === e.id){
                    return
                }
                if(data.weapon && data.weapon.id === e.id){
                    return;
                }
                if(e.isLock){
                    return;
                }

                //售价为价值的五分之一
                const gold = parseInt(elementCfgs[e.name].gold * 0.25);
                if(gold <= 0){
                    return;
                }

                html += `<div class="npc-shop-win-item">`;
                html += `<h5>${e.name}<span class="to-cybernet" data-name="${e.name}">🔍</span> * ${getNumByBag(e.name)}</h5>`;
                html += `<div class="npc-price">出价：${gold}金币</div>`;
                html += `<button class="sell-npc-item" data-itemId="${e.id}" data-id="${id}" data-num="1">卖1个</button>`;
                html += `<button class="sell-npc-item" data-itemId="${e.id}" data-id="${id}" data-num="10">卖10个</button>`;
                html += `<button class="sell-npc-item" data-itemId="${e.id}" data-id="${id}" data-num="1000000">卖全部</button>`;
                html += `</div>`;
            })
            $('.npc-shop-win-items').append(html);
        }

        $('.npc-shop-win').show();
    }
}

//点击购买NPC的商品
$('.npc-shop-win-items').on('click', '.buy-npc-item', function(){
    const id = $(this).attr('data-id');
    const name = $(this).attr('data-name');
    let num = parseInt($(this).attr('data-num'));
    const npc = data.world.npcs.filter(e => e.id === id)[0];
    const has = npc.items[name];
    const gold = parseInt(elementCfgs[name].gold * 0.8) || 1;
    //避免购买的数量超出NPC持有数量
    if(num > has){
        num = has;
    }
    let need = gold * num;

    //避免金币不足
    if(data.gold < gold){
        winMsg('金币不足', false, false)
        return;
    }
    if(data.gold < need){
        num = parseInt(data.gold / gold);
        need = gold * num
    }

    data.gold -= need;
    addItem(name, num);
    npc.gold += need;
    npc.items[name] -= num;
    //删除已经没有持有的物品
    if(npc.items[name] <= 0){
        delete npc.items[name];
    }

    winMsg(`你向${npc.name}购买了${name}*${num}`)
    save();
    renderNpcShop();
})

//点击出售物品给NPC
$('.npc-shop-win-items').on('click', '.sell-npc-item', function(){
    const id = $(this).attr('data-id');
    const itemId = $(this).attr('data-itemId');
    const item = getBagItemById(itemId);
    const name = item.name;
    let num = parseInt($(this).attr('data-num'));
    const has = getNumByBag(name);
    const npc = data.world.npcs.filter(e => e.id === id)[0];
    const gold = parseInt(elementCfgs[name].gold * 0.25);
    //避免购买的数量超出NPC持有数量
    if(num > has){
        num = has;
    }
    let need = gold * num;

    //避免金币不足
    if(npc.gold < gold){
        winMsg(`${npc.name}的金币不足`, false, false)
        return;
    }

    if(npc.gold < need){
        num = parseInt(npc.gold / gold);
        need = gold * num;
    }

    npc.gold -= need;
    reduceItemById(itemId, num);
    data.gold += need;
    if(!npc.items){
        npc.items = {};
    }
    if(!npc.items[name]){
        npc.items[name] = 0;
    }
    npc.items[name] += num;

    winMsg(`你向${npc.name}出售了${name}*${num}`)
    save();
    renderNpcShop();
})

//点击攻击NPC
$('.atk-npc-menu').click(function(){
    if(!confirm('是否确认主动发起攻击？')){
        return;
    }
    const id = $('.npc-win').attr('data-id');
    if(id) {
        const npc = data.world.npcs.filter(e => e.id === id)[0];
        startBattle(npc.profession, true, false, true, id)
        $('.npc-win-modal').hide();
        $('.npc-win').hide();
    }
});
